/*
 *  Revision
 *  - made str_cats() less error-prone, changed return type to size_t
 *  Revision 20180221, Kai Peter
 *  - renamed str_append() --> str_cats(), str_start() --> str_pcmp()
 *  - replaced 'str_copy' and 'str_diff(n)' by wrapper functions
 *  - improved function declarations
 *  Revision 20171211, Kai Peter
 *  - gcc-6 compatibility: prevent '-Wmisleading-indentation' warning
 *  Revision 20170501, Kai Peter
 *  - new function: str_append()
*/
#include "str.h"
/* Consolidate the "str_*.c" functions into one source file. Original files
   shipped with qmail-1.03.
   The "str" functions will be linked to "str.a" only!
   Included files:    Size (bytes)    Date
     - str_chr.c              350     19980615
     - str_cpy.c              330     19980615
     - str_diff.c             439     19980615
     - str_diffn.c            550     19980615
     - str_len.c              239     19980615
     - str_rchr.c             397     19980615
     - str_start.c            342     19980615                                */

/* file: str_chr.c ********************************************************** */
size_t str_chr(register char *s, char c) {
  register char ch;
  register char *t;

  ch = c;
  t = s;
  for (;;) {
    if (!*t) { break; } if (*t == ch) { break; } ++t;
    if (!*t) { break; } if (*t == ch) { break; } ++t;
    if (!*t) { break; } if (*t == ch) { break; } ++t;
    if (!*t) { break; } if (*t == ch) { break; } ++t;
  }
  return (size_t)(t - s);
}

/* file: str_cpy.c ********************************************************** */
/*
size_t str_copy(register char *s, register const char *t) {
  register int len;

  len = 0;
  for (;;) {
    if (!(*s = *t)) { break; } ++s; ++t; ++len;
    if (!(*s = *t)) { break; } ++s; ++t; ++len;
    if (!(*s = *t)) { break; } ++s; ++t; ++len;
    if (!(*s = *t)) { break; } ++s; ++t; ++len;
  }
  return (size_t)len;
}
*/
// alternative:
size_t str_copy(char *s, const char *t) { return byte_copy(s,str_len(t),t); }

/* file: str_diff.c ********************************************************* */
/*
int str_diff2(register char *s,register char *t) {
  register char x;

  for (;;) {
    x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
    x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
    x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
    x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
  }
  return ((int)(unsigned int)(unsigned char) x)
       - ((int)(unsigned int)(unsigned char) *t);
}
*/
// alternative:
/* check if s is a prefix of t (return type of byte_diff is int) */
size_t str_diff(char *s, char *t) { return (size_t)byte_diff(s,str_len(s),t); }


/* file: str_diffn.c ******************************************************** */
/*
int str_diffn2(register char *s,register char *t,size_t len) {
  register char x;

  for (;;) {
    if (!len--) { return 0; } x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
    if (!len--) { return 0; } x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
    if (!len--) { return 0; } x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
    if (!len--) { return 0; } x = *s; if (x != *t) { break; } if (!x) { break; } ++s; ++t;
  }
  return ((int)(unsigned int)(unsigned char) x)
       - ((int)(unsigned int)(unsigned char) *t);
}
*/
/* file: str_len.c ********************************************************** */
size_t str_len(register const char *s) {
  register const char *t;

  t = s;
  for (;;) {
    if (!*t) { break; } ++t;
    if (!*t) { break; } ++t;
    if (!*t) { break; } ++t;
    if (!*t) { break; } ++t;
  }
  return (size_t)(t - s);
}

/* file: str_rchr.c ********************************************************* */
size_t str_rchr(register char *s, char c) {
  register char ch;
  register char *t;
  register char *u;

  ch = c;
  t = s;
  u = 0;
  for (;;) {
    if (!*t) { break; } if (*t == ch) { u = t; } ++t;
    if (!*t) { break; } if (*t == ch) { u = t; } ++t;
    if (!*t) { break; } if (*t == ch) { u = t; } ++t;
    if (!*t) { break; } if (*t == ch) { u = t; } ++t;
  }
  if (!u) u = t;
  return (size_t)(u - s);
}

/* compare string prefixes ************************************************** */
/* This function was called 'str_start' from file str_start.c.                */
size_t str_pcmp(register char *s, register char *t) {
  register char x;

  for (;;) {
    x = *t++; if (!x) return 1; if (x != *s++) return 0;
    x = *t++; if (!x) return 1; if (x != *s++) return 0;
    x = *t++; if (!x) return 1; if (x != *s++) return 0;
    x = *t++; if (!x) return 1; if (x != *s++) return 0;
  }
}

/* str_cats: similar to the 'strcat' C standard function ******************** */
ssize_t str_cats(char **dst, char *src) {
  static stralloc sa = {0};
  ssize_t len;

  char *s = *dst;
  if (!stralloc_copys(&sa,s)) return -1;
  if (!stralloc_cats(&sa,src)) return -1;
  len = sa.len;    /* length w/o \0 */
  if (!stralloc_0(&sa)) return -1;
  *dst = sa.s;
  return (len);
}
